import { builtinModules } from 'module';
import { CallSiteLike } from './call-site-like.js';
export { CallSiteLike } from './call-site-like.js';
const regExpEscape = (s) => s.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
// platform portability accommodation
/* c8 ignore start */
let cwd = typeof process === 'object' &&
    process &&
    typeof process.cwd === 'function'
    ? process.cwd()
    : undefined;
/* c8 ignore stop */
/**
 * Set the effective cwd for shortening filenames in stack traces
 * Set to `undefined` to show full absolute paths.
 */
export const setCwd = (c) => (cwd = c);
/**
 * Get the effective cwd for shortening filenames in stack traces
 * If set to `undefined`, then will show full absolute paths.
 */
export const getCwd = () => cwd;
let filterNodeInternals = true;
/**
 * Get the current value indicating whether node internals should be
 * filtered out. (Defaults to true)
 */
export const getFilterNodeInternals = () => filterNodeInternals;
/**
 * Set whether node internals should be filtered out.
 */
export const setFilterNodeInternals = (s) => (filterNodeInternals = s);
// these are packages that function somewhat like node internals,
// for tap's purposes, and @tapjs packages themselves, when loaded
// from node_modules.
const ignoredPackages = [
    '@tapjs',
    'ts-node',
    'pirates',
    'function-loop',
    '@cspotcode/source-map-support',
    'signal-exit',
    'async-hook-domain',
];
let dirty = false;
/**
 * Add a package name to the list of deps that should be excluded
 * from stack traces.
 */
export const addIgnoredPackage = (s) => {
    const i = ignoredPackages.indexOf(s);
    if (i === -1) {
        ignoredPackages.push(s);
        dirty = true;
    }
};
/**
 * Remove a package name from the list of deps that should be excluded
 * from stack traces.
 */
export const removeIgnoredPackage = (s) => {
    const i = ignoredPackages.indexOf(s);
    if (i !== -1) {
        ignoredPackages.splice(i, 1);
        dirty = true;
    }
};
/**
 * Get a read-only copy of the list of deps that should be excluded
 * from stack traces.
 */
export const getIgnoredPackages = () => Object.freeze(ignoredPackages.slice(0));
const buildIgnoredPackages = () => {
    // just a safety precaution, no reason to ever do this
    /* c8 ignore start */
    if (!ignoredPackages.length)
        return undefined;
    /* c8 ignore stop */
    const p = ignoredPackages.map(s => regExpEscape(s));
    return new RegExp(`[/\\\\]node_modules[/\\\\](?:${p.join('|')})([/\\\\]|$)`);
};
let ignoredPackagesRE = buildIgnoredPackages();
let filterIgnoredPackages = true;
/**
 * Set whether or not the list of ignored packages should
 * be excluded from stack traces.
 */
export const setFilterIgnoredPackages = (s) => (filterIgnoredPackages = s);
/**
 * Get whether or not the list of ignored packages should
 * be excluded from stack traces.
 */
export const getFilterIgnoredPackages = () => filterIgnoredPackages;
// detect the first line of Error.stack, 'Error: blah'
const isErrorStackHead = (c) => !!c &&
    c.lineNumber === null &&
    c.columnNumber === null &&
    c.this === undefined &&
    c.evalOrigin === undefined &&
    c.function === undefined &&
    c.typeName === null &&
    c.methodName === null &&
    typeof c.functionName === 'string' &&
    c.isEval === false &&
    c.isNative === false &&
    c.isToplevel === false &&
    c.isConstructor === false &&
    c.generated === undefined;
const filter = (c) => {
    const s = c.fileName;
    // technically this is possible, but super unlikely
    /* c8 ignore start */
    if (!s)
        return true;
    /* c8 ignore stop */
    if (dirty && filterIgnoredPackages) {
        ignoredPackagesRE = buildIgnoredPackages();
        dirty = false;
    }
    return ((!filterNodeInternals ||
        !(s.startsWith('node:') || builtinModules.includes(s))) &&
        (!filterIgnoredPackages || !ignoredPackagesRE?.test(s)));
};
const clean = (c) => {
    const filtered = c.filter(filter);
    while (isErrorStackHead(filtered[0]))
        filtered.shift();
    if (cwd !== undefined) {
        for (const c of filtered) {
            c.cwd = cwd;
        }
    }
    return filtered;
};
let capturing = false;
export function capture(limit = 0, fn = capture) {
    // should be impossible to hit, but can if we have to look up
    // a sourcemap for a file we haven't seen before, and the filename
    // happens to come through as a url or relative path.
    /* c8 ignore start */
    if (capturing)
        return [];
    /* c8 ignore stop */
    capturing = true;
    if (typeof limit === 'function') {
        fn = limit;
        limit = 0;
    }
    const { prepareStackTrace, stackTraceLimit } = Error;
    Error.prepareStackTrace = CallSiteLike.prepareStackTrace;
    if (limit) {
        // we always get an extra few frames to account for internals
        // or proxy frames that might be filtered out of the top.
        Error.stackTraceLimit = limit + 10;
    }
    const obj = { stack: [] };
    Error.captureStackTrace(obj, fn);
    const { stack } = obj;
    Object.assign(Error, { prepareStackTrace, stackTraceLimit });
    const st = clean(stack);
    capturing = false;
    return limit === 0 ? st : st.slice(0, limit);
}
/**
 * Get the current callsite location, from the location specified
 * by `fn`.
 */
export const at = (fn = at) => {
    const [site] = capture(1, fn);
    return site;
};
export function captureString(limit = Infinity, fn = captureString) {
    if (typeof limit === 'function') {
        fn = limit;
        limit = 0;
    }
    const stack = capture(limit, fn);
    return (limit === 0 ? stack : stack.slice(0, limit))
        .map(c => String(c) + '\n')
        .join('');
}
/**
 * Get an array of {@link CallSiteLike} objects corresponding to the
 * stack trace of the Error object provided.
 */
export const captureError = (e) => clean(
// errors almost always have stacks
/* c8 ignore start */
(e.stack || '')
    /* c8 ignore stop */
    .trim()
    .split('\n')
    .slice(1)
    .filter(l => !!l)
    .map(line => new CallSiteLike(e, line)));
/**
 * Get a processed string stack corresponding to the
 * stack trace of the Error object provided.
 */
export const captureErrorString = (e) => captureError(e)
    .map(c => String(c) + '\n')
    .join('');
/**
 * Parse a stack string and return an array of CallSiteLike objects
 *
 * We use this to get the `at` diagnostic callsite when all we have is
 * a stack, either from a native Error object, or a stringified CallSiteLike
 * stack.
 */
export const parseStack = (s) => clean(s
    .trimEnd()
    .split('\n')
    .filter(l => !!l.trim())
    .map(line => new CallSiteLike(null, line)));
//# sourceMappingURL=index.js.map